local M = {}

local jbeamIO     = require "jbeam/io"
local jbeamLoader = require "jbeam/loader"

local getRandomConfig     = require "scripts/VehicleAndPartRandomizer/utils/getRandomConfig"
local getRandomPaint      = require "scripts/VehicleAndPartRandomizer/utils/getRandomPaint"
local getRandomParts      = require "scripts/VehicleAndPartRandomizer/utils/getRandomParts"
local getRandomVariables  = require "scripts/VehicleAndPartRandomizer/utils/getRandomVariables"
local fillSlots           = require "scripts/VehicleAndPartRandomizer/utils/fillSlots"

local nop = nop
local im = ui_imgui
local windowOpen = im.BoolPtr(false)

local windowSize = {
	width =  ((((GFXDevice or {}).getDesktopMode or nop)() or {}).width  or 2560)/16*2,
	height = ((((GFXDevice or {}).getDesktopMode or nop)() or {}).height or 1440)/9*0.8
}

local function randomizeVehicle()
	local randomConfig = getRandomConfig()
	if not randomConfig then return end

	core_vehicles.replaceVehicle(randomConfig.model_key, {config = randomConfig.key})
end

local function randomizeParts()
	local playerVehicle = getPlayerVehicle(0)
	if not playerVehicle then return end

	local randomParts = getRandomParts(playerVehicle, jbeamIO, fillSlots)
	if not randomParts then return end

	local config = {
		mainPartName = randomParts.chosenPartName,
		partsTree = randomParts,
		paints = {getRandomPaint(), getRandomPaint(), getRandomPaint()}
	}

	playerVehicle:respawn(serialize(config))
end

local function randomizePartsAndTuning()
	local playerVehicle = getPlayerVehicle(0)
	if not playerVehicle then return end

	local randomParts = getRandomParts(playerVehicle, jbeamIO, fillSlots)
	if not randomParts then return end

	local config = {
		mainPartName = randomParts.chosenPartName,
		partsTree = randomParts
	}

	local randomVariables = getRandomVariables(playerVehicle, config, jbeamLoader)
	if randomVariables then config.vars = randomVariables end

	config.paints = {getRandomPaint(), getRandomPaint(), getRandomPaint()}

	playerVehicle:respawn(serialize(config))
end

local function onUpdate(dt)
	if not windowOpen[0] then return end

	im.SetNextWindowSize(im.ImVec2(windowSize.width, windowSize.height), im.Cond_FirstUseEver)
	im.Begin("Vehicle And Part Randomizer by _N_S_ v1", windowOpen)
		local SCALE = im.GetWindowWidth() / 320
		im.SetWindowFontScale(SCALE)

		im.Button("Randomize Vehicle", im.ImVec2(im.GetContentRegionAvailWidth(), 0))
		nop(im.IsItemClicked() and randomizeVehicle())

		im.Button("Randomize Parts", im.ImVec2(im.GetContentRegionAvailWidth(), 0))
		nop(im.IsItemClicked() and randomizeParts())

		im.Button("Randomize Parts And Tuning", im.ImVec2(im.GetContentRegionAvailWidth(), 0))
		nop(im.IsItemClicked() and randomizePartsAndTuning())
	im.End()
end

local function toggleUI()
	windowOpen[0] = not windowOpen[0]
end

M.onUpdate = onUpdate
M.toggleUI = toggleUI
return M